import { CrudActions, CrudConfig } from './crud.model';
/**
 * Generates ngrx code to add, remove update, upsert entities on a list, it also
 * tracks the changes made, helpful for implementing batch updates. The `storeChanges` (false by default),
 * will store for add and update the changed entity props in the property `entityChanges` of the `Change` object.
 * @param config
 * @param config.storeChanges - Will store the changes made, default fals
 *
 * @example
 * // The following trait config
 *
 * export interface TestState
 * extends EntityAndStatusState<Todo>, CrudState<Todo>{}
 *
 *    const traits = createEntityFeatureFactory(
 *      addLoadEntities<Todo>(),
 *      addCrudEntities<Todo>()
 *    )({
 *      actionsGroupKey: '[Todos]',
 *      featureSelector: createFeatureSelector<TestState>>(
 *        'todos',
 *      ),
 *    });
 * // will generate  the following actions and selectors, plus the ones generated by other traits
 * traits.actions.add(entity1,entity2...)
 * traits.actions.update({id: id1, changes:{prop1}},{id: id2, changes:{prop2}} ...)
 * traits.actions.upsert(entity1,entity2...)
 * traits.actions.removeAll()
 * traits.actions.clearChanges()
 *
 * traits.actions.selectChanges({id})
 * traits.selectors.selectAllChanges()
 * traits.selectors.selectEntitySelected()
 * traits.selectors.selectFilteredChanges()
 * traits.selectors.selectAllFilteredChanges()
 */
export declare function addCrudEntities<Entity>({ storeChanges, }?: CrudConfig): import("ngrx-traits").TraitFactory<import("./crud.model").EntityAndCrudState<Entity>, CrudActions<Entity>, import("./crud.model").CrudSelectors<Entity>, import("./crud.model").CrudMutators<Entity>, "crud", CrudConfig, import("ngrx-traits").AllTraitConfigs>;
