import { EntityAndStatusState, LoadEntitiesConfig } from './load-entities.model';
/**
 * Generates the ngrx code needed to load a list of entities from the backend
 * with a fetch, fetchSuccess, fetchFail actions, and selectors to query the
 * entities like selectAll, selectIds, selectEntities, and its progress loading
 * with isLoading,isSuccess, isFail. This trait is the base for all other traits related
 * to a list of entities, the other will call fetch when needing data.
 * @param traitConfig - Config object fot the trait factory
 * @param traitConfig.selectId - Function that returns the id of an entity
 * @param traitConfig.sortComparer - Default sort function for to @ngrx/entity EntityAdapter
 *
 * @example
 * // The following trait config
 *
 * export interface TestState
 * extends EntityAndStatusState<Todo>{}
 *
 *    const traits = createEntityFeatureFactory(
 *      addLoadEntities<Todo>(),
 *    )({
 *      actionsGroupKey: '[Todos]',
 *      featureSelector: createFeatureSelector<TestState>>(
 *        'todos',
 *      ),
 *    });
 * // will generate the actions and selectors
 * traits.actions.fetch()
 * traits.actions.fetchSuccess({entities: todos})
 * traits.actions.fetchFail();
 * traits.selectors.selectAll
 * traits.selectors.selectEntities
 * traits.selectors.selectIds
 * traits.selectors.selectTotal
 * traits.selectors.isLoading
 * traits.selectors.isSuccess
 * traits.selectors.isFail
 */
export declare function addLoadEntities<Entity>(traitConfig?: Omit<LoadEntitiesConfig<Entity>, 'adapter'>): import("ngrx-traits").TraitFactory<EntityAndStatusState<Entity>, import("./load-entities.model").LoadEntitiesActions<Entity>, import("./load-entities.model").LoadEntitiesSelectors<Entity>, import("./load-entities.model").LoadEntitiesMutators<Entity>, "loadEntities", LoadEntitiesConfig<Entity>, import("ngrx-traits").AllTraitConfigs>;
