import { MultipleSelectionSelectors, MultiSelectActions } from './multi-selection.model';
/**
 * Generates ngrx code to add multi selection to a list
 *
 * @example
 * // The following trait config
 *
 * export interface TestState
 * extends EntityAndStatusState<Todo>,MultipleSelectionState{}
 *
 *    const traits = createEntityFeatureFactory(
 *      addLoadEntities<Todo>(),
 *      addMultiSelection<Todo>()
 *    )({
 *      actionsGroupKey: '[Todos]',
 *      featureSelector: createFeatureSelector<TestState>>(
 *        'todos',
 *      ),
 *    });
 * // will generate  the following actions and selectors, plus the ones generated by other traits
 * traits.actions.multiSelect({id})
 * traits.actions.multiDeselect({id})
 * traits.actions.multiToggleSelect({id})
 * traits.actions.toggleSelectAll()
 * traits.actions.multiClearSelection()
 *
 * traits.selectors.selectIdSelected()
 * traits.selectors.selectEntitySelected()
 */
export declare function addMultiSelection<Entity>(): import("ngrx-traits").TraitFactory<import("./multi-selection.model").EntityAndMultipleSelectionState<Entity>, MultiSelectActions, MultipleSelectionSelectors<Entity>, import("./multi-selection.model").MultipleSelectionMutators<Entity>, "multiSelection", unknown, import("ngrx-traits").AllTraitConfigs>;
