import { SingleSelectionActions, SingleSelectionConfig } from './single-selection.model';
/**
 * Generates ngrx code to add single selection to a list
 * @param config
 * @param config.selectedId - Default selected id
 *
 * @example
 * // The following trait config
 *
 * export interface TestState
 * extends EntityAndStatusState<Todo>,SingleSelectionState{}
 *
 *    const traits = createEntityFeatureFactory(
 *      addLoadEntities<Todo>(),
 *      addSingleSelection<Todo>()
 *    )({
 *      actionsGroupKey: '[Todos]',
 *      featureSelector: createFeatureSelector<TestState>>(
 *        'todos',
 *      ),
 *    });
 * // will generate  the following actions and selectors, plus the ones generated by other traits
 * traits.actions.select({id})
 * traits.actions.deselect()
 * traits.actions.toggleSelect({id})
 * traits.selectors.selectIdSelected()
 * traits.selectors.selectEntitySelected()
 */
export declare function addSingleSelection<Entity>(config?: SingleSelectionConfig): import("ngrx-traits").TraitFactory<import("./single-selection.model").EntityAndSingleSelectionState<Entity>, SingleSelectionActions, import("./single-selection.model").SingleSelectionSelectors<Entity>, import("./single-selection.model").SingleSelectionMutators<unknown>, "singleSelection", SingleSelectionConfig, import("ngrx-traits").AllTraitConfigs>;
