import { SortActions, SortConfig, SortSelectors } from './sort.model';
/**
 * Generates ngrx code to sort locally or remotely a list of entities
 * @param config
 * @param config.defaultSort - Required, Default entity prop for the sort
 * @param config.remote - Optional field, default to false, when true disables local
 * sorting and every sort action call will now trigger a fetch action and the backend
 * should sort the data, use selectSort in the effect that call backend to get the requested sort,
 * when false all sorting is done in memory when the sort action is fired
 *
 * @example
 * // The following trait config
 * export interface TestState
 * extends EntityAndStatusState<Todo>, SortState<Todo>{}
 *
 *    const traits = createEntityFeatureFactory(
 *      addLoadEntities<Todo>(),
 *      addSort<Todo>({
 *        defaultSort: {active:'id', direction:'asc'}
 *      })
 *    )({
 *      actionsGroupKey: '[Todos]',
 *      featureSelector: createFeatureSelector<TestState>>(
 *        'todos',
 *      ),
 *    });
 * // will generate  the following actions and selectors, plus the ones generated by other traits
 * traits.actions.sort({active:'id', direction:'desc'})
 * traits.actions.resetSort()
 *
 * traits.selectors.selectSort()
 */
export declare function addSort<Entity>({ remote, defaultSort, }: SortConfig<Entity>): import("ngrx-traits").TraitFactory<import("./sort.model").EntityAndSortState<Entity>, SortActions<Entity>, SortSelectors<Entity>, import("./sort.model").SortMutators<Entity>, "sort", SortConfig<Entity>, import("ngrx-traits").AllTraitConfigs>;
